<?php
/**
*
* @package phpBB Extension - Like&Dislike
* @copyright (c) 2019 southklad
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace southklad\likedislike\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
* Event listener
*/
class listener implements EventSubscriberInterface
{
/**
* Assign functions defined in this class to event listeners in the core
*
* @return array
* @static
* @access public
*/
	static public function getSubscribedEvents()
	{
		return array(
			'core.user_setup'							=> 'load_language_on_setup',
			'core.viewtopic_get_post_data'				=> 'get_post_data',
			'core.viewtopic_modify_post_row'			=> 'modify_post_row',
			'core.viewtopic_post_rowset_data'			=> 'get_rowset_data',
			'core.handle_post_delete_conditions'		=> 'post_delete',
			'core.viewforum_modify_topicrow'			=> 'modify_topicrow',
			//'core.viewforum_get_topic_ids_data'			=> 'get_topic_ids_data',
		);
	}

	/** @var \phpbb\db\driver\driver */
	protected $db;

	//** @var string phpbb_root_path */
	protected $phpbb_root_path;

	/** @var \phpbb\controller\helper */
	protected $helper;

	/** @ \southklad\likedislike\Tables */
	protected $rating_table;

	/** @ \southklad\likedislike\Tables */
	protected $rating_users_table;

	/** @ \southklad\likedislike\Tables */
	protected $rating_topics_table;

	/**
	* Constructor
	*/
	public function __construct(
		\phpbb\auth\auth $auth,
		\phpbb\config\config $config,
		\phpbb\db\driver\driver_interface $db,
		\phpbb\user $user,
		\phpbb\template\template $template,
		\phpbb\controller\helper $helper,
		$rating_table,
		$rating_users_table,
		$rating_topics_table,
		$rc_helper
	)
	{
		$this->auth = $auth;
		$this->config = $config;
		$this->db = $db;
		$this->user = $user;
		$this->template = $template;
		$this->helper = $helper;
		$this->rating_table = $rating_table;
		$this->rating_users_table = $rating_users_table;
		$this->rating_topics_table = $rating_topics_table;
		$this->rc_helper = $rc_helper;
	}

	public function load_language_on_setup($event)
	{
		$lang_set_ext = $event['lang_set_ext'];
		$lang_set_ext[] = array(
			'ext_name' => 'southklad/likedislike',
			'lang_set' => 'common',
		);
		$event['lang_set_ext'] = $lang_set_ext;
	}

	public function modify_topicrow($event)
	{
		$row = $event['row'];
		$topic_id = $event['row']['topic_id'];
		$topic_first_post_id = $event['row']['topic_first_post_id'];
		$sql = 'SELECT positiv, negativ
			FROM ' . $this->rating_table . '
				WHERE post_id = ' . $topic_first_post_id;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$first_post_positiv = $row['positiv'] ?? 0;
		$first_post_negativ = $row['negativ'] ?? 0;
		$first_post_total = $first_post_positiv + $first_post_negativ;
		$this->db->sql_freeresult($result);

		$sql = 'SELECT positiv, negativ
			FROM ' . $this->rating_topics_table . '
				WHERE topic_rate_id = ' . $topic_id;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$positiv = $row['positiv'] ?? 0;
		$negativ = $row['negativ'] ?? 0;
		$total = $positiv + $negativ;
		$this->db->sql_freeresult($result);
		$event['topic_row'] = array_merge($event['topic_row'], array(
			'POSITIV'				=> $positiv,
			'NEGATIV'				=> $negativ,
			'TOTAL'					=> $total,
			'FIRST_POST_POSITIV'	=> $first_post_positiv,
			'FIRST_POST_NEGATIV'	=> $first_post_negativ,
			'FIRST_POST_TOTAL'		=> $first_post_total,
		));
	}

	public function post_delete($event)
	{
		if ($event['mode'] != 'is_soft')
		{
			$this->rc_helper->udate_topic_rating($event['post_id'], $event['topic_id']);
		}
	}

	public function get_post_data($event)
	{
		$post_list = $event['post_list'];
		$sql_ary = $event['sql_ary'];
		$select = $sql_ary['SELECT'] . ', ru.positiv, ru.negativ, pu.time_rate, pu.rate_user_id, pu.positiv AS plus, pu.negativ AS minus';
		$from = $sql_ary['FROM'];
		$left_join = array_merge($sql_ary['LEFT_JOIN'], array
			(
				array(
					'FROM'	=> array('' . $this->rating_table . '' => 'ru'),
					'ON'	=> 'p.post_id = ru.post_id',
				),

				array(
					'FROM'	=> array('' . $this->rating_users_table . '' => 'pu'),
					'ON'	=> 'pu.rate_user_id = ' . $this->user->data['user_id'] . ' AND pu.post_rate_id = p.post_id',
				),
			)
		);
		$where = $sql_ary['WHERE'];
		$sql_ary = array();
		$sql_ary = array_merge($sql_ary, array('SELECT' => $select), array('FROM' => $from), array('LEFT_JOIN' => $left_join), array('WHERE' => $where));

		$event['sql_ary'] = $sql_ary;
	}

	public function modify_post_row($event)
	{
		$rate_enable = false;
		$row = $event['row'];
		$post_rating = $event['row']['positiv'] + $event['row']['negativ'];
		$plus = $event['row']['plus'];
		$minus = $event['row']['minus'];
		$time = $event['row']['time_rate'];
		$rate_user_id = $event['row']['rate_user_id'];
		$post_id = $event['row']['post_id'];
		if (($this->user->data['user_id'] != ANONYMOUS && $this->user->data['user_id'] != $event['row']['user_id']))
		{
			$rate_enable = true;
		}

		$event['post_row'] = array_merge($event['post_row'], array(
			'POST_ID'			=> $event['row']['post_id'],
			'POST_RATE'			=> $post_rating,
			'S_POSITIV'			=> ($post_rating >= 0) ? true : false,
			'POST_RATE_NEGATIV'	=> ($event['row']['negativ']) ? $event['row']['negativ'] : 0,
			'POST_RATE_POSITIV'	=> ($event['row']['positiv']) ? $event['row']['positiv'] : 0,
			'U_POST_RATE_UP'	=> (!$plus && $rate_enable) ? $this->helper->route('southklad_likedislike_add_rate_controller', array('post_id' => $event['row']['post_id'], 'type_id' => 1)) : '',
			'U_POST_RATE_DOWN'	=> (!$minus && $rate_enable) ? $this->helper->route('southklad_likedislike_add_rate_controller', array('post_id' => $event['row']['post_id'], 'type_id' => 0)) : '',
			'U_POST_RATE_CLEAR'	=> ($this->auth->acl_get('a_')) ? $this->helper->route('southklad_likedislike_add_rate_controller', array('post_id' => $event['row']['post_id'], 'type_id' => 2)) : '',
			)
		);
	}

	public function get_rowset_data($event)
	{
		$event['rowset_data'] = array_merge($event['rowset_data'],
			array(
				'positiv'		=> $event['row']['positiv'],
				'negativ'		=> $event['row']['negativ'],
				'time_rate'		=> $event['row']['time_rate'],
				'rate_user_id'	=> $event['row']['rate_user_id'],
				'plus'			=> $event['row']['plus'],
				'minus'			=> $event['row']['minus'],
			)
		);
	}
}
